package de.devbliss.paulparser;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.Locale;
import java.util.Map;
import java.util.logging.Logger;
import java.util.regex.Pattern;

/**
 * Attempts to parse a date by using regexs to try and work out what sort of date it is, then
 * parsing it.
 */
public class FuzzyDateParser {
    private static final Map<Pattern, String> DATE_FORMATS = new LinkedHashMap<Pattern, String>();

    static Logger logger = Logger.getLogger(FuzzyDateParser.class.getCanonicalName());
    
    static {
        // 2010-08-24
        DATE_FORMATS.put(Pattern.compile("\\d{4}-\\d\\d?-\\d\\d?"), "yyyy-M-d");
        // 2010/08/24
        DATE_FORMATS.put(Pattern.compile("\\d{4}/\\d\\d?/\\d\\d?"), "yyyy/M/d");
        // 24 August 2010
        DATE_FORMATS.put(Pattern.compile("\\d\\d? [a-zA-Z]{3,} \\d{4}"), "d MMM yyyy");
        // 24. August 2010
        DATE_FORMATS.put(Pattern.compile("\\d\\d?\\. [a-zA-Z]{3,} \\d{4}"), "d. MMM yyyy");
        // August 24 2010
        DATE_FORMATS.put(Pattern.compile("[a-zA-Z]{3,} \\d\\d? \\d{4}"), "MMM d yyyy");
        // August 24, 2010
        DATE_FORMATS.put(Pattern.compile("[a-zA-Z]{3,} \\d\\d?, \\d{4}"), "MMM d, yyyy");
        // 2010 August 24
        DATE_FORMATS.put(Pattern.compile("\\d{4} [a-zA-Z]{3,} \\d\\d?"), "yyyy MMM dd");
        // August 2010
        DATE_FORMATS.put(Pattern.compile("[a-zA-Z]{3,} \\d{4}"), "MMM yyyy");
        // August, 2010
        DATE_FORMATS.put(Pattern.compile("[a-zA-Z]{3,}, \\d{4}"), "MMM, yyyy");
        // 2010 August
        DATE_FORMATS.put(Pattern.compile("\\d{4} [a-zA-Z]{3,}"), "yyyy MMM");
        // 2010
        DATE_FORMATS.put(Pattern.compile("\\d{4}"), "yyyy");
    }

    public static Date parse(String date) {
        // First sanitise white space
        date = date.trim().replaceAll("\\p{javaSpaceChar}+", " ");
        // Now find a matching entry
        for (Map.Entry<Pattern, String> entry : DATE_FORMATS.entrySet()) {
            if (entry.getKey().matcher(date).matches()) {
                // It matches, try and parse
                try {
                    return new SimpleDateFormat(entry.getValue(), Locale.US).parse(date);
                } catch (ParseException pe) {
                    
                    logger.warning("Input date \"" + date + "\" was not parseable by pattern "
                            + entry.getValue());
                    return null;
                }
            }
        }
        logger.warning("No suitable date pattern found to parse date: \"" + date + "\"");
        return null;
    }
}
